#!/bin/bash
# -----------------------------------------------------------------------------------------
#  Example Installation Script Template
#  
#  This convenience script encapsulates command-line instructions highlighted in
#  the OpenHPC Install Guide that can be used as a starting point to perform a local
#  cluster install beginning with bare-metal. Necessary inputs that describe local
#  hardware characteristics, desired network settings, and other customizations
#  are controlled via a companion input file that is used to initialize variables 
#  within this script.
#   
#  Please see the OpenHPC Install Guide for more information regarding the
#  procedure. Note that the section numbering included in this script refers to
#  corresponding sections from the install guide.
#  
#  OpenHPC 15.42
# -----------------------------------------------------------------------------------------

inputFile=${OHPC_INPUT_LOCAL:-/opt/ohpc/pub/doc/recipes/vanilla/input.local}

if [ ! -e ${inputFile} ];then
   echo "Error: Unable to access local input file -> ${inputFile}"
   exit 1
else
   . ${inputFile} || { echo "Error sourcing ${inputFile}"; exit 1; }
fi

# ---------------------------- Begin OpenHPC Recipe ---------------------------------------
# Commands below are extracted from an OpenHPC install guide recipe and are intended for 
# execution on the master SMS host.
# -----------------------------------------------------------------------------------------

# ------------------------------------------------------------
# Add baseline OpenHPC and provisioning services (Section 3.3)
# ------------------------------------------------------------
zypper -n install -t pattern ohpc-base
zypper -n install -t pattern ohpc-warewulf
# Disable firewall 
systemctl disable SuSEfirewall2
systemctl stop SuSEfirewall2
# Enable NTP services on SMS host
systemctl enable ntpd.service
echo "server ${ntp_server}" >> /etc/ntp.conf
systemctl restart ntpd

# -------------------------------------------------------------
# Add resource management services on master node (Section 3.4)
# -------------------------------------------------------------
zypper -n install -t pattern ohpc-slurm-server
useradd slurm

# ------------------------------------------------------------
# Add InfiniBand support services on master node (Section 3.5)
# ------------------------------------------------------------
zypper -n install libibverbs-runtime libmlx4-rdmav2 libipathverbs-rdmav2
zypper -n install libibmad5 librdmacm1 rdma infinipath-psm dapl-devel dapl-utils 
cp /opt/ohpc/pub/examples/udev/60-ipath.rules /etc/udev/rules.d/
systemctl start rdma

if [[ ${enable_ipoib} -eq 1 ]];then
     # Enable ib0
     cp /opt/ohpc/pub/examples/network/sles/ifcfg-ib0 /etc/sysconfig/network
     perl -pi -e "s/master_ipoib/${sms_ipoib}/" /etc/sysconfig/network/ifcfg-ib0
     perl -pi -e "s/ipoib_netmask/${ipoib_netmask}/" /etc/sysconfig/network/ifcfg-ib0
     ifup ib0
fi

# -----------------------------------------------------------
# Complete basic Warewulf setup for master node (Section 3.6)
# -----------------------------------------------------------
perl -pi -e "s/device = eth1/device = ${sms_eth_internal}/" /etc/warewulf/provision.conf
perl -pi -e "s/^DHCPD_INTERFACE=\S+/DHCPD_INTERFACE=${sms_eth_internal}/" /etc/sysconfig/dhcpd
perl -pi -e "s/^\s+disable\s+= yes/ disable = no/" /etc/xinetd.d/tftp
perl -pi -e "s#\#tftpdir = /var/lib/#tftpdir = /srv/#" /etc/warewulf/provision.conf
export MODFILE=/etc/apache2/conf.d/warewulf-httpd.conf
perl -pi -e "s#modules/mod_perl.so\$#/usr/lib64/apache2/mod_perl.so#" $MODFILE
perl -pi -e "s/cgi-bin>\$/cgi-bin>\n Require all granted/" $MODFILE
perl -pi -e "s/Allow from all/Require all granted/" $MODFILE
perl -ni -e "print unless /^\s+Order allow,deny/" $MODFILE
ifconfig ${sms_eth_internal} ${sms_ip} netmask ${internal_netmask} up
systemctl restart xinetd
systemctl enable mysql.service
systemctl restart mysql
systemctl enable apache2.service
systemctl restart apache2
if [ ! -z ${BOS_MIRROR+x} ]; then
     perl -pi -e "s#^ZYP_MIRROR=(\S+)#ZYP_MIRROR=${BOS_MIRROR}#" /usr/libexec/warewulf/wwmkchroot/sles-12.tmpl
fi

# -------------------------------------------------
# Create compute image for Warewulf (Section 3.7.1)
# -------------------------------------------------
export CHROOT=/opt/ohpc/admin/images/sles12sp1
mkdir -p -m 755 $CHROOT                        # create chroot housing dir
mkdir -m 755 $CHROOT/dev                       # create chroot /dev dir
mknod -m 666 $CHROOT/dev/zero c 1 5            # create /dev/zero device
wwmkchroot sles-12 $CHROOT                     # create base image

# -------------------------------------------------------
# Add OpenHPC components to compute image (Section 3.7.2)
# -------------------------------------------------------
cp -p /etc/resolv.conf $CHROOT/etc/resolv.conf
zypper -n --root $CHROOT --no-gpg-checks --gpg-auto-import-keys refresh
zypper -n --root $CHROOT install -t pattern ohpc-slurm-client
zypper -n --root $CHROOT install libibverbs-runtime libmlx4-rdmav2 libipathverbs-rdmav2
zypper -n --root $CHROOT install libibmad5 librdmacm1 rdma infinipath-psm dapl-devel dapl-utils
cp /opt/ohpc/pub/examples/udev/60-ipath.rules $CHROOT/etc/udev/rules.d/
zypper -n --root $CHROOT install ntp
zypper -n --root $CHROOT install lmod-ohpc
chroot $CHROOT systemctl enable sshd.service
mv $CHROOT/etc/hostname $CHROOT/etc/hostname.orig

# ----------------------------------------------
# Customize system configuration (Section 3.7.3)
# ----------------------------------------------
wwinit ssh_keys
cat ~/.ssh/cluster.pub >> $CHROOT/root/.ssh/authorized_keys
echo "${sms_ip}:/home /home nfs nfsvers=3,rsize=1024,wsize=1024,cto 0 0" >> $CHROOT/etc/fstab
echo "${sms_ip}:/opt/ohpc/pub /opt/ohpc/pub nfs nfsvers=3 0 0" >> $CHROOT/etc/fstab
perl -pi -e "s/ControlMachine=\S+/ControlMachine=${sms_name}/" /etc/slurm/slurm.conf
echo "/home *(rw,no_subtree_check,fsid=10,no_root_squash)" >> /etc/exports
echo "/opt/ohpc/pub *(ro,no_subtree_check,fsid=11)" >> /etc/exports
exportfs -a
systemctl restart nfsserver
chroot $CHROOT systemctl enable ntpd
echo "server ${sms_ip}" >> $CHROOT/etc/ntp.conf
cp /etc/ntp.keys $CHROOT/etc/ntp.keys

# Update basic slurm configuration if additional computes defined
if [ ${num_computes} -gt 4 ];then
   perl -pi -e "s/^NodeName=(\S+)/NodeName=c[1-${num_computes}]/" /etc/slurm/slurm.conf
   perl -pi -e "s/^PartitionName=normal Nodes=(\S+)/PartitionName=normal Nodes=c[1-${num_computes}]/" /etc/slurm/slurm.conf
   perl -pi -e "s/^NodeName=(\S+)/NodeName=c[1-${num_computes}]/" $CHROOT/etc/slurm/slurm.conf
   perl -pi -e "s/^PartitionName=normal Nodes=(\S+)/PartitionName=normal Nodes=c[1-${num_computes}]/" $CHROOT/etc/slurm/slurm.conf
fi

# -----------------------------------------
# Additional customizations (Section 3.7.4)
# -----------------------------------------
echo "* soft memlock unlimited" >> /etc/security/limits.conf
echo "* hard memlock unlimited" >> /etc/security/limits.conf
echo "* soft memlock unlimited" >> $CHROOT/etc/security/limits.conf
echo "* hard memlock unlimited" >> $CHROOT/etc/security/limits.conf
# Enable slurm pam module
echo "account    required     pam_slurm.so" >> $CHROOT/etc/pam.d/sshd

# ---------------------------------------------------------
# Add Cluster Checker to SMS and computes (Section 3.7.4.3)
# ---------------------------------------------------------
zypper -n install intel-clck-ohpc
zypper -n --root $CHROOT install intel-clck-ohpc

if [[ ${enable_lustre_client} -eq 1 ]];then
     # Install Lustre client on master
     zypper -n install lustre-client-ohpc lustre-client-ohpc-modules
     perl -pi -e "s/^allow_unsupported_modules 0/allow_unsupported_modules 1/" /etc/modprobe.d/10-unsupported-modules.conf
     # Enable lustre in WW compute image
     zypper -n --root $CHROOT install lustre-client-ohpc lustre-client-ohpc-modules
     perl -pi -e "s/^allow_unsupported_modules 0/allow_unsupported_modules 1/" $CHROOT/etc/modprobe.d/10-unsupported-modules.conf
     mkdir $CHROOT/mnt/lustre
     echo "${mgs_fs_name} /mnt/lustre lustre defaults,_netdev,localflock 0 0" >> $CHROOT/etc/fstab
     # Enable o2ib for Lustre
     echo "options lnet networks=o2ib(ib0)" >> /etc/modprobe.d/lustre.conf
     echo "options lnet networks=o2ib(ib0)" >> $CHROOT/etc/modprobe.d/lustre.conf
     # mount Lustre client on master
     mkdir /mnt/lustre
     mount -t lustre -o localflock ${mgs_fs_name} /mnt/lustre
fi


if [[ ${enable_nagios} -eq 1 ]];then
     # Install Nagios on master and vnfs image
     zypper -n install -t pattern ohpc-nagios
     zypper -n --root $CHROOT install -t pattern ohpc-nagios
     chroot $CHROOT systemctl enable nrpe
     perl -pi -e "s/^allowed_hosts=/# allowed_hosts=/" $CHROOT/etc/nagios/nrpe.cfg
     echo "nrpe 5666/tcp # NRPE"         >> $CHROOT/etc/services
     echo "nrpe : ${sms_ip}  : ALLOW"    >> $CHROOT/etc/hosts.allow
     echo "nrpe : ALL : DENY"            >> $CHROOT/etc/hosts.allow
     chroot $CHROOT /usr/sbin/useradd -c "NRPE user for the NRPE service" -d /var/run/nrpe -r -g nrpe -s /sbin/nologin nrpe
     chroot $CHROOT /usr/sbin/groupadd -r nrpe
     mv /etc/nagios/conf.d/services.cfg.example /etc/nagios/conf.d/services.cfg
     mv /etc/nagios/conf.d/hosts.cfg.example /etc/nagios/conf.d/hosts.cfg
     for ((i=0; i<$num_computes; i++)) ; do
        perl -pi -e "s/HOSTNAME$(($i+1))/${c_name[$i]}/ || s/HOST$(($i+1))_IP/${c_ip[$i]}/" \
        /etc/nagios/conf.d/hosts.cfg
     done
     perl -pi -e "s/ \/bin\/mail/ \/usr\/bin\/mailx/g" /etc/nagios/objects/commands.cfg
     perl -pi -e "s/nagios\@localhost/root\@${sms_name}/" /etc/nagios/objects/contacts.cfg
     echo command[check_ssh]=/usr/lib64/nagios/plugins/check_ssh localhost >> $CHROOT/etc/nagios/nrpe.cfg
     chkconfig nagios on
     systemctl start nagios
     chmod u+s `which ping`
fi

if [[ ${enable_ganglia} -eq 1 ]];then
     # Install Ganglia on master
     zypper -n install -t pattern ohpc-ganglia
     zypper -n --root $CHROOT install ganglia-gmond-ohpc
     cp /opt/ohpc/pub/examples/ganglia/gmond.conf /etc/ganglia/gmond.conf
     perl -pi -e "s/<sms>/${sms_name}/" /etc/ganglia/gmond.conf
     cp /etc/ganglia/gmond.conf $CHROOT/etc/ganglia/gmond.conf
     echo "gridname MySite" >> /etc/ganglia/gmetad.conf
     systemctl enable gmond
     systemctl enable gmetad
     systemctl start gmond
     systemctl start gmetad
     chroot $CHROOT systemctl enable gmond
     systemctl try-restart httpd
fi

     # Install clustershell
     zypper -n install clustershell-ohpc
     cd /etc/clustershell/groups.d
     mv local.cfg local.cfg.orig
     echo "adm: ${sms_name}" > local.cfg
     echo "compute: c[1-${num_computes}]" >> local.cfg
     echo "all: @adm,@compute" >> local.cfg

if [[ ${enable_mrsh} -eq 1 ]];then
     # Install mrsh
     zypper -n install mrsh-ohpc mrsh-rsh-compat-ohpc
     zypper -n --root $CHROOT install mrsh-ohpc mrsh-rsh-compat-ohpc mrsh-server-ohpc
     echo "mshell          21212/tcp                  # mrshd" >> /etc/services
     echo "mlogin            541/tcp                  # mrlogind" >> /etc/services
     chroot $CHROOT systemctl enable xinetd
fi

if [[ ${enable_genders} -eq 1 ]];then
     # Install genders
     zypper -n install genders-ohpc
     echo -e "${sms_name}\tsms" > /etc/genders
     for ((i=0; i<$num_computes; i++)) ; do
        echo -e "${c_name[$i]}\tcompute,bmc=${c_bmc[$i]}"
     done >> /etc/genders
fi

if [[ ${enable_powerman} -eq 1 ]];then
     # Optionally, install powerman
     zypper -n install powerman-ohpc
     cp /etc/powerman/powerman.conf{.example,}
     perl -pi -e 's/^\#(tcpwrappers yes)/$1/' /etc/powerman/powerman.conf
     perl -pi -e 's/^\#(include "\/etc\/powerman\/ipmipower\.dev")/$1/' /etc/powerman/powerman.conf
     for ((i=0; i<$num_computes; i++)) ; do
        perl -pi -e 'print "device \"ipmi'$i'\" \"ipmipower\" \"/usr/sbin/ipmipower -h ".
        "'${c_bmc[$i]}' -u '$bmc_username' -p '${IPMI_PASSWORD:-undefined}'|&\"\n" if(/^\#device "ipmi1"/);' \
        /etc/powerman/powerman.conf
     done
     for ((i=0; i<$num_computes; i++)) ; do
        perl -pi -e 'print "node \"'${c_name[$i]}'\" \"ipmi'$i'\" \"'${c_bmc[$i]}'\"\n"
        if(/^\#node "t1"/);' /etc/powerman/powerman.conf
     done
     systemctl enable powerman
     systemctl start powerman
     pm -q
fi

# --------------------------------------------------------
# Configure rsyslog on SMS and computes (Section 3.7.4.11)
# --------------------------------------------------------
perl -pi -e "s/\\#\\\$ModLoad imudp/\\\$ModLoad imudp/" /etc/rsyslog.conf
perl -pi -e "s/\\#\\\$UDPServerRun 514/\\\$UDPServerRun 514/" /etc/rsyslog.conf
systemctl restart rsyslog
echo "*.* @${sms_ip}:514" >> $CHROOT/etc/rsyslog.conf
perl -pi -e "s/^\*\.info/\\#\*\.info/" $CHROOT/etc/rsyslog.conf
perl -pi -e "s/^authpriv/\\#authpriv/" $CHROOT/etc/rsyslog.conf
perl -pi -e "s/^mail/\\#mail/" $CHROOT/etc/rsyslog.conf
perl -pi -e "s/^cron/\\#cron/" $CHROOT/etc/rsyslog.conf
perl -pi -e "s/^uucp/\\#uucp/" $CHROOT/etc/rsyslog.conf

# ----------------------------
# Import files (Section 3.7.5)
# ----------------------------
wwsh file import /etc/passwd
wwsh file import /etc/group
wwsh file import /etc/shadow 
wwsh file import /etc/slurm/slurm.conf
wwsh file import /etc/munge/munge.key

if [[ ${enable_ipoib} -eq 1 ]];then
     wwsh file import /opt/ohpc/pub/examples/network/sles/ifcfg-ib0.ww
     wwsh -y file set ifcfg-ib0.ww --path=/etc/sysconfig/network/ifcfg-ib0
fi

# --------------------------------------
# Assemble bootstrap image (Section 3.8)
# --------------------------------------
export WW_CONF=/etc/warewulf/bootstrap.conf
echo "drivers += updates/kernel/" >> $WW_CONF
wwbootstrap `uname -r`
# Assemble VNFS
wwvnfs -y --chroot $CHROOT
# Add hosts to cluster
wwsh -y node new ${c_name[0]} --ipaddr=${c_ip[0]} --hwaddr=${c_mac[0]} -D ${eth_provision}
wwsh -y node new ${c_name[1]} --ipaddr=${c_ip[1]} --hwaddr=${c_mac[1]} -D ${eth_provision}
wwsh -y node new ${c_name[2]} --ipaddr=${c_ip[2]} --hwaddr=${c_mac[2]} -D ${eth_provision}
wwsh -y node new ${c_name[3]} --ipaddr=${c_ip[3]} --hwaddr=${c_mac[3]} -D ${eth_provision}
# Add hosts to cluster (Cont.)
for ((i=4; i<$num_computes; i++)) ; do
   wwsh -y node new ${c_name[$i]} --ipaddr=${c_ip[$i]} --hwaddr=${c_mac[$i]} -D ${eth_provision}
done
wwsh -y provision set "${compute_regex}" --vnfs=sles12sp1 --bootstrap=`uname -r` --files=dynamic_hosts,passwd,group,shadow,slurm.conf,munge.key

# Optionally, define IPoIB network settings (required if planning to mount Lustre over IB)
if [[ ${enable_ipoib} -eq 1 ]];then
     for ((i=0; i<$num_computes; i++)) ; do
        wwsh -y node set ${c_name[$i]} -D ib0 --ipaddr=${c_ipoib[$i]} --netmask=${ipoib_netmask}
     done
     wwsh -y provision set "${compute_regex}" --fileadd=ifcfg-ib0.ww
fi


# Optionally, define diskless node kernel arguments to support SOL console
if [[ ${enable_ipmisol} -eq 1 ]];then
     zypper -n install conman-ohpc
     for ((i=0; i<$num_computes; i++)) ; do
        echo -n 'CONSOLE name="'${c_name[$i]}'" dev="ipmi:'${c_bmc[$i]}'" '
        echo 'ipmiopts="'U:${bmc_username},P:${IPMI_PASSWORD:-undefined},W:solpayloadsize'"'
     done >> /etc/conman.conf
     systemctl enable conman
     systemctl start conman
     wwsh -y provision set "${compute_regex}" --kargs "${kargs} console=ttyS1,115200"
fi
# Optionally, add arguments to bootstrap kernel
if [[ ${enable_kargs} ]]; then
   wwsh provision set "${compute_regex}" --kargs=${kargs}
fi

# --------------------------------
# Boot compute nodes (Section 3.9)
# --------------------------------
ipmitool -E -I lanplus -H ${c_bmc[0]} -U ${bmc_username} chassis power reset   # power cycle c1
ipmitool -E -I lanplus -H ${c_bmc[1]} -U ${bmc_username} chassis power reset   # power cycle c2
ipmitool -E -I lanplus -H ${c_bmc[2]} -U ${bmc_username} chassis power reset   # power cycle c3
ipmitool -E -I lanplus -H ${c_bmc[3]} -U ${bmc_username} chassis power reset   # power cycle c4
for ((i=4; i<$num_computes; i++)) ; do
   ipmitool -E -I lanplus -H ${c_bmc[$i]} -U ${bmc_username} chassis power reset
done

# ---------------------------------------
# Install Development Tools (Section 4.1)
# ---------------------------------------
zypper -n install -t pattern ohpc-autotools
zypper -n install valgrind-ohpc
zypper -n install EasyBuild-ohpc
zypper -n install spack-ohpc
zypper -n install R_base-ohpc            

# -------------------------------
# Install Compilers (Section 4.2)
# -------------------------------
zypper -n install gnu-compilers-ohpc intel-compilers-devel-ohpc

# --------------------------------
# Install MPI Stacks (Section 4.3)
# --------------------------------
zypper -n install openmpi-*-ohpc mvapich2-*-ohpc intel-mpi-ohpc
zypper -n install -t pattern ohpc-imb

# ---------------------------------------
# Install Performance Tools (Section 4.4)
# ---------------------------------------
zypper -n install papi-ohpc
zypper -n install intel-itac-ohpc
zypper -n install intel-vtune-ohpc
zypper -n install intel-advisor-ohpc
zypper -n install intel-inspector-ohpc
zypper -n install -t pattern ohpc-mpiP
zypper -n install -t pattern ohpc-tau
zypper -n install lmod-defaults-gnu-mvapich2-ohpc

# ---------------------------------------------------
# Install 3rd Party Libraries and Tools (Section 4.6)
# ---------------------------------------------------
zypper -n install -t pattern ohpc-adios        
zypper -n install -t pattern ohpc-boost        
zypper -n install -t pattern ohpc-fftw         
zypper -n install -t pattern ohpc-gsl          
zypper -n install -t pattern ohpc-hdf5         
zypper -n install -t pattern ohpc-hypre        
zypper -n install -t pattern ohpc-metis        
zypper -n install -t pattern ohpc-mumps        
zypper -n install -t pattern ohpc-netcdf       
zypper -n install -t pattern ohpc-numpy        
zypper -n install -t pattern ohpc-ocr
zypper -n install -t pattern ohpc-openblas     
zypper -n install -t pattern ohpc-petsc        
zypper -n install -t pattern ohpc-phdf5        
zypper -n install -t pattern ohpc-scalapack    
zypper -n install -t pattern ohpc-scipy        
zypper -n install -t pattern ohpc-trilinos     

# ------------------------------------
# Resource Manager Startup (Section 5)
# ------------------------------------
systemctl enable munge
systemctl enable slurmctld
systemctl start munge
systemctl start slurmctld
pdsh -w c[1-4] systemctl start munge
pdsh -w c[1-4] systemctl start slurmd
scontrol update nodename=c[1-4] state=idle

# Startup slurm on additional computes if defined
if [ ${num_computes} -gt 4 ];then
   pdsh -w c[5-$num_computes] systemctl start munge
   pdsh -w c[5-$num_computes] systemctl start slurmd
fi
useradd -m test
wwsh file resync passwd shadow group
pdsh -w c[1-4] /warewulf/bin/wwgetfiles 
